/*
 * date.js - Implementation of useful date functions
 * Copyright (C) 2002-2005, Alex Eng <ateng@users.sourceforge.net>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//
// The original code in this library originated from Acme StartingPoint, an
// MSHTA application I single-handedly created in the summer of 2001, shortly 
// after being coldly rejected from Crystal Decisions (which has since been 
// bought by Business Objects SA) for a technical writer position.  
// The MSHTA app was never released to the public, due to various problems 
// with performance and dependence on Microsoft's now-defunct Java VM.
//


/*
 * NEW - returns a string containing the date expressed in the format of the
 * XML Schema `dateTime' data type.
 * W3C reference: <http://www.w3.org/TR/xmlschema-2/#dateTime>
 *
 * Format: yyyy-mm-ddThh:mm:ss (the 'T' in the middle is a literal character)
 */
function dtGetXSDDateTimeString(oDate) {
  var dateStr = dtGetDateString(oDate);
  var timeStr = dtGetTimeString(oDate);
  var rv = dateStr + "T" + timeStr;
  return rv;
}


/*
 * Returns the time in the format "hh:mm:ss"
 */
function dtGetTimeString(oDate) {
  var hrs = oDate.getHours();
  if (hrs < 10) hrs = "0" + hrs;
  var min = oDate.getMinutes();
  if (min < 10) min = "0" + min;
  var sec = oDate.getSeconds();
  if (sec < 10) sec = "0" + sec;
  return hrs + ":" + min + ":" + sec;
}



const DATEFMT_AMERICAN = 1;
const DATEFMT_EUROPEAN = 2;

/*
 * Returns a string containing the date expressed in the given format.
 */
function dtGetDateString(oDate, nFormat) {
  var month = oDate.getMonth();
  var date = oDate.getDate();
  var year = oDate.getFullYear();
  var sDate;

  month += 1;

  if (nFormat == DATEFMT_AMERICAN)
    sDate = month + "/" + date + "/" + year;
  else if (nFormat == DATEFMT_EUROPEAN)
    sDate = date + "/" + month + "/" + year;
  else {
    if (month < 10) month = "0" + month;  
    if (date < 10) date = "0" + date;

    sDate = year + "-" + month + "-" + date;
  }
  return sDate;
}




/*
 * Returns a Date object containing the date expressed in the given date
 * string and in the given date format.
 * If the given date string is invalid (e.g. mixed separators), then 
 * null is returned.
 */
function dtGetDateObject(sDate, nFormat) {
  var year, month, date, oDate;

  if (nFormat == DATEFMT_AMERICAN) {
    oDate = new Date(sDate);
  }
  else if (nFormat == DATEFMT_EUROPEAN) {
    // sDate is in the format "d/m/yyyy"
    // sDateAndMonth is in the format "d/m"
    var sDateAndMonth = sDate.substring(0, sDate.lastIndexOf("/"));

    year  = sDate.substring((sDate.lastIndexOf("/") + 1), sDate.length);
    month = sDateAndMonth.substring((sDateAndMonth.indexOf("/") + 1),
				    sDateAndMonth.length);
    date  = sDateAndMonth.substring(0, sDateAndMonth.indexOf("/"));

    oDate = new Date(year, month, date);
  }
  else {
    year  = sDate.substr(0, 4);
    month = sDate.substr(5, 2);
    date  = sDate.substr(8, 2);
    month--;  // 0 = January, 1 = February, 2 = March, ... , 11 = December

    oDate = new Date(year, month, date);
  }

  return oDate;
}



/*
 * Verifies the validity of the given date string.
 */
function dtIsValidDateString(sDate, nFormat) {
  // Steps for validating date string:
  // 1. Check for correct separator character.
  // 2. Check that the month, date and year values are in the correct ranges.

  var year, month, date;
  var nNumSeparators = 0;

  if (nFormat == DATEFMT_AMERICAN) {
    for (var i = 0; i < sDate.length; i++) {
      if (sDate.charAt(i) == "/")
	nNumSeparators++;
    }

    if (nNumSeparators != 2) return false;

    // sDate is in the format "m/d/yyyy"
    // sMonthAndDate is in the format "m/d"
    var sMonthAndDate = sDate.substring(0, sDate.lastIndexOf("/"));
    
    year  = sDate.substring((sDate.lastIndexOf("/") + 1), sDate.length);
    month = sMonthAndDate.substring(0, sMonthAndDate.indexOf("/"));
    date  = sMonthAndDate.substring((sMonthAndDate.indexOf("/") + 1),
				    sMonthAndDate.length);
  }
  else if (nFormat == DATEFMT_EUROPEAN) {
    for (var j = 0; j < sDate.length; j++) {
      if (sDate.charAt(j) == "/")
	nNumSeparators++;
    }

    if (nNumSeparators != 2) return false;

    // sDate is in the format "d/m/yyyy"
    // sDateAndMonth is in the format "d/m"
    var sDateAndMonth = sDate.substring(0, sDate.lastIndexOf("/"));

    year  = sDate.substring((sDate.lastIndexOf("/") + 1), sDate.length);
    month = sDateAndMonth.substring((sDateAndMonth.indexOf("/") + 1),
				    sDateAndMonth.length);
    date  = sDateAndMonth.substring(0, sDateAndMonth.indexOf("/"));
  }
  else {
    for (var k = 0; k < sDate.length; k++) {
      if (sDate.charAt(k) == "-")
	nNumSeparators++;
    }

    if (nNumSeparators != 2) return false;

    year  = sDate.substr(0, 4);
    month = sDate.substr(5, 2);
    date  = sDate.substr(8, 2);
  }

  return (year > 0 && month <= 12 && month >= 1 && date >= 1 && date <= 31);
}




/*
 * Returns the day of the week (Monday, Tuesday, Wednesday, ...) 
 */
function dtGetDayOfTheWeek(oDate) {
  var aDays = new Array("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday");

  return (aDays[oDate.getDay()]);
}




/*
 * Returns the month represented by the given Date object.
 */
function dtGetMonth(oDate) {
  var aMonths = new Array("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December");

  return (aMonths[oDate.getMonth()]);
}



/*
 * Returns a string containing the full date.
 */
function dtGetFriendlyDate(oDate) {
  var dayOfWeek = oDate.getDay();   // The day of the week (Monday, Tuesday...)
  var thisMonth = oDate.getMonth(); // The current month 
  var nDayOfTheMonth = oDate.getDate(); // Day of the month (1, 2, ..., 30, 31)

  switch (dayOfWeek) {
  case 0:
    dayOfWeek = "Sunday";
    break;
  case 1:
    dayOfWeek = "Monday";
    break;
  case 2:
    dayOfWeek = "Tuesday";
    break;
  case 3:
    dayOfWeek = "Wednesday";
    break;
  case 4:
    dayOfWeek = "Thursday";
    break;
  case 5:
    dayOfWeek = "Friday";
    break;
  case 6:
    dayOfWeek = "Saturday";
    break;
  default:
    break;
  }

  switch (thisMonth) {
  case 0:
    thisMonth = "January";
    break;
  case 1:
    thisMonth = "February";
    break;
  case 2: 
    thisMonth = "March";
    break;
  case 3: 
    thisMonth = "April";
    break;
  case 4: 
    thisMonth = "May";
    break;
  case 5: 
    thisMonth = "June";
    break;
  case 6: 
    thisMonth = "July";
    break;
  case 7: 
    thisMonth = "August";
    break;
  case 8: 
    thisMonth = "September";
    break;
  case 9: 
    thisMonth = "October";
    break;
  case 10: 
    thisMonth = "November";
    break;
  case 11: 
    thisMonth = "December";
    break;
  default:
    break;
  }

  // Calculate the year.
  var year = oDate.getFullYear();

  return (dayOfWeek + ", " + thisMonth + " " + nDayOfTheMonth + ", " + year);

}




/*
 * Returns the number of days in the given month (0 = January, 1 = February,
 * 2 = March, ... , 11 = December)
 */
function dtNumDaysInMonth(nMonth, nFullYear) {
  var nNumDays = null;

  switch (nMonth) {
  case 0: case 2: case 4: case 6: case 7: case 9: case 11:
    nNumDays = 31;
    break;
  case 3: case 5: case 8: case 10:
    nNumDays = 30;
    break;
  case 1:  // February
    nNumDays = dtIsLeapYear(nFullYear) ? 29 : 28;
    break;
  default:
    break;
  }

  return nNumDays;
}



/* 
 * Returns true if the given month (0 = January, 1 = February, ... , 11 = Dec.)
 * has 31 days, false if it does not, or null if the given month is invalid.
 */
function dtHas31Days(nMonth) {
  var retval = null;

  switch (nMonth) {
  case 0: case 2: case 4: case 6: case 7: case 9: case 11:
    retval = true;
    break;
  case 1: case 3: case 5: case 8: case 10:
    retval = false;
    break;
  default:
    break;
  }

  return retval;
}



/*
 * Returns true if the given 4-digit year is a leap year.
 */
function dtIsLeapYear(nFullYear) {
  return ((nFullYear % 100 != 0 && nFullYear % 4 == 0) 
	  || (nFullYear % 100 == 0 && nFullYear % 400 == 0));
}



/*
 * Returns true if the two Date objects represent the same date (year,
 * month and day), false otherwise.
 */
function dtIsSameDate(oDate1, oDate2) {
  return (oDate1.getDay() == oDate2.getDay() 
	  && oDate1.getMonth() == oDate2.getMonth()
	  && oDate1.getDate()  == oDate2.getDate()
	  && oDate1.getFullYear() == oDate2.getFullYear());
}
